/* * * * * * * * * * * * * *  INITIALIZATIONS  * * * * * * * * * * * * * *\
 * findgfx () [char/boolean]                                             *
 * gfxmode () [void]                                                     *
 * [global struct] VgaInfoBlock, ModeInfoBlock, memModels                *
 * [global var] xres, yres, bytesperline, curBank, bankShift, oldMode    *
 * [global var] bankSwitch [void _far (*) (void)]                        *
 * [global var] screenPtr [char _far *]                                  *
 *   see below for details                                               *
 *                                                                       *
 * * * * * * * * * * * * * * * *  DRAWING  * * * * * * * * * * * * * * * *
 * pset (x,y,color) [macro to dpset]                                     *
 * point (x,y) [unsigned char asm]                                       *
 * dpset (x,y,color) [void macro asm]                                    *
 *                                                                       *
 * * * * * * * * * * * * * * * PAGE FLIPPING * * * * * * * * * * * * * * *
 * flip () [null]                                                        *
 *                                                                       *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


// ========================== INITIALIZATIONS ==========================
#undef MAXX
#define MAXX 640
#undef MAXY
#define MAXY 480
typedef struct {   // sVGA info
    char    VESASignature[4];       // 'VESA' 4 byte signature
    short   VESAVersion;            // VBE version number
    char    _far *OEMStringPtr;     // Pointer to OEM string
    long    Capabilities;           // Capabilities of video card
    short   _far *VideoModePtr;     // Pointer to supported modes
    short   TotalMemory;            // Number of 64kb memory blocks
    char    reserved[236];          // Pad to 256 byte block size
  } VgaInfoBlock;
typedef struct {   // sVGA mode info
    short   ModeAttributes;         // Mode attributes
    char    WinAAttributes;         // Window A attributes
    char    WinBAttributes;         // Window B attributes
    short   WinGranularity;         // Window granularity in k
    short   WinSize;                // Window size in k
    short   WinASegment;            // Window A segment
    short   WinBSegment;            // Window B segment
    void    _far *WinFuncPtr;       // Pointer to window function
    short   BytesPerScanLine;       // Bytes per scanline
    short   XResolution;            // Horizontal resolution
    short   YResolution;            // Vertical resolution
    char    XCharSize;              // Character cell width
    char    YCharSize;              // Character cell height
    char    NumberOfPlanes;         // Number of memory planes
    char    BitsPerPixel;           // Bits per pixel
    char    NumberOfBanks;          // Number of CGA style banks
    char    MemoryModel;            // Memory model type
    char    BankSize;               // Size of CGA style banks
    char    NumberOfImagePages;     // Number of images pages
    char    res1;                   // Reserved
    char    RedMaskSize;            // Size of direct color red mask
    char    RedFieldPosition;       // Bit posn of lsb of red mask
    char    GreenMaskSize;          // Size of direct color green mask
    char    GreenFieldPosition;     // Bit posn of lsb of green mask
    char    BlueMaskSize;           // Size of direct color blue mask
    char    BlueFieldPosition;      // Bit posn of lsb of blue mask
    char    RsvdMaskSize;           // Size of direct color res mask
    char    RsvdFieldPosition;      // Bit posn of lsb of res mask
    char    DirectColorModeInfo;    // Direct color mode attributes
    char    res2[216];              // Pad to 256 byte block size
  } ModeInfoBlock;
typedef enum {
    memPL       = 3,                // Planar memory model
    memPK       = 4,                // Packed pixel memory model
    memRGB      = 6,                // Direct color RGB memory model
    memYUV      = 7,                // Direct color YUV memory model
  } memModels;
int     xres, yres;                 // Resolution of video mode used
int     bytesperline;               // Logical CRT scanline length
int     curBank;                    // Current read/write bank
int     bankShift;                  // Bank granularity adjust factor
int     oldMode;                    // Old video mode number
void    _far (*bankSwitch)(void);   // Direct bank switching function
char    _far *screenPtr;            // Pointer to start of video memory

#include <dos.h>
// can't seem to excerpt int86 and int86x.
/*#ifndef __DOS_H
#define FP_SEG( fp )( (unsigned )( void _seg * )( void far * )( fp ))
#define FP_OFF( fp )( (unsigned )( fp ))
struct WORDREGS { unsigned int ax, bx, cx, dx, si, di, cflag, flags; };
struct BYTEREGS { unsigned char al, ah, bl, bh, cl, ch, dl, dh; };
union  REGS     { struct WORDREGS x; struct BYTEREGS h; };
struct SREGS    { unsigned int es; unsigned int cs;
                  unsigned int ss; unsigned int ds; };
int         cdecl int86( int __intno,
                         union REGS *__inregs,
                         union REGS *__outregs );
int         cdecl int86x( int __intno,
                          union REGS *__inregs,
                          union REGS *__outregs,
                          struct SREGS *__segregs );
#endif*/


int getVgaInfo (VgaInfoBlock *vgaInfo) {
  union REGS regs;
  struct SREGS sregs;
  regs.x.ax = 0x4F00;
  regs.x.di = FP_OFF(vgaInfo);
  sregs.es = FP_SEG(vgaInfo);
  int86x(0x10, &regs, &regs, &sregs);
  return (regs.x.ax == 0x4F);
}

int getModeInfo (int mode, ModeInfoBlock *modeInfo) {
  union REGS regs;
  struct SREGS sregs;
  if (mode < 0x100) return 0;     // Ignore non-VBE modes
  regs.x.ax = 0x4F01;
  regs.x.cx = mode;
  regs.x.di = FP_OFF(modeInfo);
  sregs.es = FP_SEG(modeInfo);
  int86x(0x10, &regs, &regs, &sregs);
  if (regs.x.ax != 0x4F) return 0;
  if ((modeInfo->ModeAttributes & 0x1) && modeInfo->MemoryModel == memPK
    && modeInfo->BitsPerPixel == 8 && modeInfo->NumberOfPlanes == 1)
    return 1;
  return 0;
}

void setVBEMode (int mode) {
  union REGS regs;
  regs.x.ax = 0x4F02; regs.x.bx = mode;
  int86(0x10,&regs,&regs);
}

int getVBEMode (void) {
  union REGS regs;
  regs.x.ax = 0x4F03;
  int86 (0x10, &regs, &regs);
  return regs.x.bx;
}

char findgfx (void) {
  short           *p;
  VgaInfoBlock    vgaInfo;
  ModeInfoBlock   modeInfo;
  if (!getVgaInfo(&vgaInfo)) return (0); // no VESA VBE detected
  for (p = vgaInfo.VideoModePtr; *p != -1; p++)
    if (getModeInfo(*p, &modeInfo) && modeInfo.XResolution == MAXX
      && modeInfo.YResolution == MAXY)
      return (1);
  return (0);
}

void gfxmode (void) {
  short           *p;
  VgaInfoBlock    vgaInfo;
  ModeInfoBlock   modeInfo;
  getVgaInfo(&vgaInfo);
  for (p = vgaInfo.VideoModePtr; *p != -1; p++) {
    if (getModeInfo(*p, &modeInfo) && modeInfo.XResolution == MAXX
      && modeInfo.YResolution == MAXY) {
      xres = MAXX; yres = MAXY;
      bytesperline = modeInfo.BytesPerScanLine;
      bankShift = 0;
      while ((64 >> bankShift) != modeInfo.WinGranularity)
        bankShift++;
      bankSwitch = (void(far*)()) modeInfo.WinFuncPtr;
      curBank = -1;
      screenPtr = (char*) ((void _seg*)(0xA000) + (void near*)(0));
      oldMode = getVBEMode();
      setVBEMode (*p);
      return;
    }
  }
}

void txtmode (void) {
  asm { mov ax, 0x03; int 0x10; }
}

// ============================== DRAWING ==============================
void setBank (int bank) {
  if (bank == curBank) return;    // Bank is already active
  curBank = bank;                 // Save current bank number
  bank <<= bankShift;             // Adjust to window granularity
  asm { mov bx, 0; mov dx, bank }
  bankSwitch();
  asm { mov bx, 1; mov dx, bank }
  bankSwitch();
}
#define pset dpset
#define xpset dpset
unsigned char point (int x, int y) {
    long addr = (long) y * bytesperline + x;
    setBank ((int) addr >> 16);
    return *(screenPtr + ((int) addr & 0xFFFF));
}
void dpset (int x, int y, char color) {
  long addr = (long)y * bytesperline + x;
  setBank((int) addr >> 16);
  *(screenPtr + ((int) addr & 0xFFFF)) = color;
}

// =========================== PAGE FLIPPING ===========================
void flip (void) {}

#include <pjsgfx.h>
